/// <reference path="Grib2Data.ts" />
/// <reference path="Interpolate.ts" />
"use strict";
var Grib2Data_1 = require('./Grib2Data');
var Interpolate_1 = require('./Interpolate');
function floorMod(a, n) {
    var f = a - n * Math.floor(a / n);
    // HACK: when a is extremely close to an n transition, f can be equal to n. This is bad because f must be
    //       within range [0, n). Check for this corner case. Example: a:=-1e-16, n:=10. What is the proper fix?
    return f === n ? 0 : f;
}
function isValue(x) {
    return x != null && x != undefined;
}
function makeField(header, grid, interpolate) {
    var λ0 = header.lo1;
    var φ0 = header.la1;
    var Δλ = header.dx;
    var Δφ = header.dy;
    return function (λ, φ) {
        var i = floorMod(λ - λ0, 360) / Δλ;
        var j = (φ0 - φ) / Δφ;
        var fi = Math.floor(i);
        var ci = fi + 1;
        var fj = Math.floor(j);
        var cj = fj + 1;
        var row;
        if ((row = grid[fj])) {
            var g00 = row[fi];
            var g10 = row[ci];
            if (isValue(g00) && isValue(g10) && (row = grid[cj])) {
                var g01 = row[fi];
                var g11 = row[ci];
                if (isValue(g01) && isValue(g11)) {
                    // All four points found, so interpolate the value.
                    return interpolate({
                        x: i - fi,
                        y: j - fj,
                        g00: g00,
                        g10: g10, g01: g01, g11: g11
                    });
                }
            }
        }
        console.warn("cannot interpolate: " + λ + "," + φ + ": " + fi + " " + ci + " " + fj + " " + cj);
        return null;
    };
}
function makeGrid(accessor, ni, nj, isContinuous) {
    var grid = [];
    var p = 0;
    for (var j = 0; j < nj; ++j) {
        var row = [];
        for (var i = 0; i < ni; ++i, ++p) {
            row[i] = accessor(p);
        }
        if (isContinuous) {
            row.push(row[0]);
        }
        grid[j] = row;
    }
    return grid;
}
function build(data, interpolateFunc) {
    if (interpolateFunc === void 0) { interpolateFunc = Interpolate_1.bilinearInterpolateVector; }
    var header1 = data[0].header;
    var accessor = function (idx) { return data.map(function (segment) { return segment.data[idx]; }); };
    var dataSource = Grib2Data_1.getDataSource(header1);
    var date = new Date(header1.refTime);
    date.setHours(date.getHours() + header1.forecastTime);
    var isContinuous = Math.floor(header1.nx * header1.dx) >= 360;
    var grid = makeGrid(accessor, header1.nx, header1.ny, isContinuous);
    var field = makeField(header1, grid, interpolateFunc);
    return {
        grid: grid,
        date: date,
        dataSource: dataSource,
        field: field
    };
}
exports.build = build;
//# sourceMappingURL=Grid.js.map